import React, { useReducer } from 'react';
import { Box, Text, Newline } from 'ink';
import { InitialProjectConfiguration, } from './ProjectState.js';
import { StepDataProvider } from './StepDataProvider.js';
import { StepAuthProvider } from './StepAuthProvider.js';
import { StepResources } from './StepResources.js';
import { StepInstall } from './StepInstall.js';
import { StepName } from './StepName.js';
import { StepGenerate } from './StepGenerate.js';
import { StepRunInstall } from './StepRunInstall.js';
const getNextStep = (state) => {
    if (state.name) {
        if (state.dataProvider) {
            if (state.authProvider) {
                if (state.resources) {
                    if (state.installer) {
                        return 'generate';
                    }
                    return 'install';
                }
                return 'resources';
            }
            return 'auth-provider';
        }
        return 'data-provider';
    }
    return 'name';
};
const stepReducer = (state, action) => {
    switch (state.step) {
        case 'name': {
            const newState = {
                ...state,
                name: action.value,
            };
            return {
                ...newState,
                step: getNextStep(newState),
            };
        }
        case 'data-provider': {
            const newState = {
                ...state,
                dataProvider: action.value,
                authProvider: action.value === 'ra-supabase'
                    ? 'none'
                    : state.authProvider,
                resources: action.value === 'ra-data-fakerest' &&
                    (state.resources == null || state.resources.length === 0)
                    ? ['posts', 'comments']
                    : state.resources,
            };
            return {
                ...newState,
                step: getNextStep(newState),
            };
        }
        case 'auth-provider': {
            const newState = {
                ...state,
                authProvider: action.value,
            };
            return {
                ...newState,
                step: getNextStep(newState),
            };
        }
        case 'resources': {
            const newState = {
                ...state,
                resources: action.value,
            };
            return {
                ...newState,
                step: getNextStep(newState),
            };
        }
        case 'install':
            const newState = {
                ...state,
                installer: action.value,
            };
            return {
                ...newState,
                step: getNextStep(newState),
            };
        case 'generate':
            return {
                ...state,
                messages: action.value.messages,
                step: state.installer && state.installer !== 'skip'
                    ? 'run-install'
                    : 'finish',
            };
        case 'run-install':
            return {
                ...state,
                step: 'finish',
            };
        default:
            return state;
    }
};
export default function App(props) {
    const sanitizedName = sanitizeName(props.name);
    const initialState = {
        ...InitialProjectConfiguration,
        dataProvider: props.dataProvider,
        authProvider: props.authProvider,
        resources: props.resources?.includes('skip')
            ? []
            : props.dataProvider === 'ra-data-fakerest' &&
                (props.resources == null || props.resources.length === 0)
                ? ['posts', 'comments']
                : props.resources,
        installer: props.install,
        name: sanitizedName,
    };
    const initialStep = getNextStep(initialState);
    const [state, dispatch] = useReducer(stepReducer, {
        ...initialState,
        step: initialStep,
    });
    const handleSubmit = (value) => {
        dispatch({ value });
    };
    if (state.step === 'name') {
        return React.createElement(StepName, { initialValue: state.name, onSubmit: handleSubmit });
    }
    if (state.step === 'data-provider') {
        return React.createElement(StepDataProvider, { onSubmit: handleSubmit });
    }
    if (state.step === 'auth-provider') {
        return React.createElement(StepAuthProvider, { onSubmit: handleSubmit });
    }
    if (state.step === 'resources') {
        return React.createElement(StepResources, { onSubmit: handleSubmit });
    }
    if (state.step === 'install') {
        return React.createElement(StepInstall, { onSubmit: handleSubmit });
    }
    if (state.step === 'generate') {
        return React.createElement(StepGenerate, { config: state, onCompleted: handleSubmit });
    }
    if (state.step === 'run-install') {
        return React.createElement(StepRunInstall, { config: state, onCompleted: handleSubmit });
    }
    let installerCommand;
    switch (state.installer) {
        case 'yarn':
            installerCommand = 'yarn';
            break;
        case 'bun':
            installerCommand = 'bun run';
            break;
        default:
            installerCommand = 'npm run';
            break;
    }
    return (React.createElement(React.Fragment, null,
        React.createElement(Box, { marginBottom: 1, marginTop: 1 },
            React.createElement(Text, null,
                "Your application ",
                React.createElement(Text, { bold: true }, state.name),
                " was successfully generated.")),
        React.createElement(Text, null,
            "To start working, run ",
            React.createElement(Text, { bold: true },
                "cd ",
                state.name),
            "."),
        state.installer ? (React.createElement(Text, null,
            "Start the app in development mode by running",
            ' ',
            React.createElement(Text, { bold: true },
                installerCommand,
                " dev"),
            ".")) : (React.createElement(Box, null,
            React.createElement(Text, null,
                "Install the dependencies using your favorite package manager.",
                React.createElement(Newline, null),
                "Run the ",
                React.createElement(Text, { bold: true }, "dev"),
                " command to start the app."))),
        React.createElement(Box, { marginBottom: 1 }, state.messages.map((line, index) => (React.createElement(Text, { key: index }, line))))));
}
const sanitizeName = (name) => {
    return name
        ? name
            .trim()
            .toLowerCase()
            .replace(/\s+/g, '-')
            .replace(/^[._]/, '')
            .replace(/[^a-z\d\-~]+/g, '-')
        : undefined;
};
//# sourceMappingURL=app.js.map